/**
 * @file
 */

(($, Drupal, cookies) => {
  // Marker today.
  function markerToday() {
    let dateToStr = gantt.date.date_to_str(gantt.config.task_date);

    let id = gantt.addMarker({
      start_date: new Date(),
      css: "today",
      title: dateToStr(new Date())
    });
    setInterval(function () {
      let today = gantt.getMarker(id);
      if (today) {
        today.start_date = new Date();
        today.title = dateToStr(today.start_date);
        gantt.updateMarker(id);
      }
    }, 1000 * 60);
    //Make resize marker for two columns
    gantt.attachEvent("onColumnResizeStart", function (ind, column) {
      if (!column.tree || ind == 0) {
        return;
      }

      setTimeout(function () {
        let marker = document.querySelector(".gantt_grid_resize_area");
        if (!marker) {
          return;
        }
        let cols = gantt.getGridColumns();
        let delta = cols[ind - 1].width || 0;
        if (!delta) {
          return;
        }

        marker.style.boxSizing = "content-box";
        marker.style.marginLeft = -delta + "px";
        marker.style.paddingRight = delta + "px";
      }, 1);
    });
  }

  // Undo Redo.
  function undo() {
    gantt.templates.scale_cell_class = function (date) {
      if (!gantt.isWorkTime(date)) {
        return "weekend";
      }
    };
    gantt.templates.timeline_cell_class = function (item, date) {
      if (!gantt.isWorkTime(date)) {
        return "weekend";
      }
    };
    gantt.attachEvent("onBeforeAutoSchedule", function () {
      gantt.message("Recalculating project schedule...");
      return true;
    });
    gantt.attachEvent("onAfterTaskAutoSchedule", function (task, new_date, constraint, predecessor) {
      gantt.message({
        text: "<b>" + task.text + "</b> has been rescheduled to " + gantt.templates.task_date(new_date) + " due to <b>" + predecessor.text + "</b> constraint",
        expire: 4000
      });
    });
  }

  // Expand at bottom.
  function expand() {
    gantt.attachEvent("onCollapse", function () {
      var icon = gantt.toggleIcon;
      if (icon) {
        icon.className = icon.className.replace("fullscreen-exit", "bi-arrows-fullscreen");
      }
    });
    gantt.attachEvent("onExpand", function () {
      var icon = gantt.toggleIcon;
      if (icon) {
        icon.className = icon.className.replace("bi-arrows-fullscreen", "fullscreen-exit");
      }
    });
    gantt.attachEvent("onTemplatesReady", function () {
      var toggle = document.createElement("i");
      toggle.className = "bi bi-arrows-fullscreen gantt-fullscreen";
      gantt.toggleIcon = toggle;
      gantt.$container.appendChild(toggle);
      toggle.onclick = function () {
        gantt.ext.fullscreen.toggle();
      };
    });
  }

  // User owner.
  function byId(list, id) {
    for (let i = 0; i < list.length; i++) {
      if (list[i].key == parseInt(id)) {
        return list[i].label || "";
      }
    }
    return "";
  }

  function owners(owner) {
    gantt.form_blocks["multiselect"] = {
      render: function (sns) {
        var height = (sns.height || "23") + "px";
        var html = "<div class='gantt_cal_ltext gantt_cal_chosen gantt_cal_multiselect' style='height:" + height + ";'><select data-placeholder='...' class='chosen-select' multiple>";
        if (sns.options) {
          for (var i = 0; i < sns.options.length; i++) {
            if(sns.unassigned_value !== undefined && sns.options[i].key == sns.unassigned_value){
              continue;
            }
            html += "<option value='" + sns.options[i].key + "'>" + sns.options[i].label + "</option>";
          }
        }
        html += "</select></div>";
        return html;
      },

      set_value: function (node, value, ev, sns) {
        node.style.overflow = "visible";
        node.parentNode.style.overflow = "visible";
        node.style.display = "inline-block";
        var select = $(node.firstChild);

        if (value) {
          value = (value + "").split(",");
          select.val(value);
        }
        else {
          select.val([]);
        }

        select.chosen();
        if (sns.onchange) {
          select.change(function () {
            sns.onchange.call(this);
          })
        }
        select.trigger('chosen:updated');
        select.trigger("change");
      },

      get_value: function (node, ev) {
        var value = $(node.firstChild).val();
        return value;
      },

      focus: function (node) {
        $(node.firstChild).focus();
      }
    };

    gantt.locale.labels.column_owner =
    gantt.locale.labels.section_owner = Drupal.t("Owner");
    gantt.config.resource_property = "staff";
    gantt.serverList("staff", owner);

    gantt.attachEvent("onTaskCreated", function (task) {
      task.staff = [];
      task.calendar_id = null;
      return true;
    });
    gantt.config.columns.unshift({
      name: "owner", width: 80, align: "center", template: (task) => {
        if (task.type == gantt.config.types.project) {
          return "";
        }
        var result = "";
        let owners = task.owner;
        if (!owners || !owners.length) {
          return "Unassigned";
        }
        if(owners.length == 1){
          return byId(gantt.serverList('staff'), task.owner[0]);
        }
        owners.forEach( (ownerId) => {
          let owner = byId(gantt.serverList('staff'), ownerId);
          if (!owner){
            return;
          }
          result += "<div class='owner-label' title='" + owner + "'>" + owner.charAt(0).toUpperCase() + "</div>";
        });
        return result;
      }, resize: true
    });

    gantt.config.lightbox.sections.push(
      {name:"owner",height:60, type:"multiselect", options:gantt.serverList("staff"), map_to:"owner"}
    );

    gantt.templates.grid_row_class =
      gantt.templates.task_row_class =
        gantt.templates.task_class = function (start, end, task) {
          let css = [];
          if (task.$virtual || task.type == gantt.config.types.project) {
            css.push("summary-bar");
          }

          if (task.owner) {
            css.push("gantt_resource_task gantt_resource_" + task.owner);
          }

          return css.join(" ");
        };

    gantt.attachEvent("onDataRender", function () {
      let styleId = "dynamicGanttStyles";
      let element = document.getElementById(styleId);
      if (!element) {
        element = document.createElement("style");
        element.id = styleId;
        document.querySelector("head").appendChild(element);
      }
      let html = [];
      let resources = gantt.serverList("staff");

      resources.forEach(function (r) {
        html.push(".gantt_task_line.gantt_resource_" + r.key + "{" +
          "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
        html.push(".gantt_row.gantt_resource_" + r.key + " .gantt_cell:nth-child(1) .gantt_tree_content{" +
          "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
      });
      element.innerHTML = html.join("");
    });

  }

  // Tooltips.
  function setTooltip() {
    let formatter = gantt.ext.formatters.durationFormatter({
      enter: "day",
      store: "hour",
      format: "day",
      hoursPerDay: 24,
      hoursPerWeek: 40,
      daysPerMonth: 30,
      short: true
    });
    let linksFormatter = gantt.ext.formatters.linkFormatter({durationFormatter: formatter});

    gantt.config.lightbox.sections = [
      {name: "description", height: 70, map_to: "text", type: "textarea", focus: true},
      {
        name: "type", type: "typeselect", map_to: "type", filter: function (name, value) {
          return !!(value != gantt.config.types.project);
        }
      },
      {name: "time", type: "duration", map_to: "auto", formatter: formatter}
    ];
    gantt.config.lightbox.project_sections = [
      {name: "description", height: 70, map_to: "text", type: "textarea", focus: true},

      {name: "time", type: "duration", readonly: true, map_to: "auto", formatter: formatter}
    ];
    gantt.config.lightbox.milestone_sections = [
      {name: "description", height: 70, map_to: "text", type: "textarea", focus: true},
      {
        name: "type", type: "typeselect", map_to: "type", filter: function (name, value) {
          return !!(value != gantt.config.types.project);
        }
      },
      {name: "time", type: "duration", single_date: true, map_to: "auto", formatter: formatter}
    ];

    gantt.templates.tooltip_text = function (start, end, task) {
      let html = "<b>" + Drupal.t('Task') + ":</b> " + task.text + "<br/><b>" + Drupal.t("Start") +
        ":</b> " + gantt.templates.tooltip_date_format(start) +
        "<br/><b>" + Drupal.t("End") + ":</b> " + gantt.templates.tooltip_date_format(end) +
        "<br><b>" + Drupal.t("Duration") + ":</b> " + formatter.format(task.duration);
      return html;
    };
  }

  function upload(file, callback, urlImport) {
    gantt.importFromMSProject({
      data: file,
      callback: function (project) {
        if (project) {
          gantt.clearAll();
          if (project.config.duration_unit) {
            gantt.config.duration_unit = project.config.duration_unit;
          }
          // mark tasks import.
          for (let i = 0; i < project.data.data.length; i++) {
            project.data.data[i].isImport = true;
          }
          gantt.parse(project.data);
          $.post(urlImport, project.data).done(function (data) {
            console.log("Data import: " + data);
          });
          if (callback) {
            callback(project);
          }
        }
      }
    });
  }

  function importMMP(ganttId, urlImport) {
    // Import microsoft project.
    let fileDnD = fileDragAndDrop();
    fileDnD.init(gantt.$container);

    function sendFile(file) {
      fileDnD.showUpload();
      upload(file, function () {
        fileDnD.hideOverlay();
      }, urlImport)
    }

    fileDnD.onDrop(sendFile);
    // Manual Upload file MPP.
    $("#mspImportBtn" + ganttId).click(function (event) {
      event.preventDefault();
      let fileInput = document.getElementById("mspFile" + ganttId);
      if (fileInput.files[0]) {
        sendFile(fileInput.files[0]);
      }
    });
  }

  function native_dialog(setting) {
    gantt.attachEvent("onTaskDblClick", function (id, e) {
      if (id == null) {
        return false;
      }
      task = gantt.getTask(id);
      if (task['type'] == 'project') {
        return false;
      }
      else {
        let ajaxSettings = {
          url: '/paragraphs_item/' + id + '/edit?' + setting.current_path,
          dialogType: 'modal',
          dialog: { width: '80%' },
        };
        Drupal.ajax(ajaxSettings).execute();
        return false;
      }
    });

    gantt.attachEvent("onBeforeLightbox", function (id) {
      let task = gantt.getTask(id);
      if (task == null) {
        return false;
      }
      if (task.$new) {
        if (setting.add_task !== true) {
          gantt.deleteTask(task.id);
          return false;
        }
        let ajaxSettings = {
          url: setting.add_link,
          dialogType: 'modal',
          dialog: {width: '80%'},
        };
        Drupal.ajax(ajaxSettings).execute();
        window.location = setting.add_link + '&parent=' + task.parent;
        gantt.deleteTask(task.id);
        return false;
      }
      return true;
    });

  }

  Drupal.behaviors.paragraphs_gantt = {
    attach: (context, settings) => {
      $(once('paragraphs_gantt', 'body', context)).each(function () {
        if (settings.gantt) {
          $(document).ready(function () {
            var paragraphGantt = [];
            var isFirst = true;

            gantt.plugins({
              fullscreen: true,
              marker: true,
              critical_path: true,
              auto_scheduling: true,
              tooltip: true,
              multiselect: true,
              undo: true
            });

            markerToday();
            expand();
            setTooltip();
            undo();

            gantt.config.auto_types = true;
            gantt.config.date_format = settings.date_format;
            gantt.config.date_grid = settings.date_format;
            gantt.config.duration_unit = "day";
            gantt.config.order_branch = "marker";
            gantt.config.order_branch_free = true;
            gantt.config.grid_resize = true;

            gantt.config.auto_scheduling = true;
            gantt.config.auto_scheduling_strict = true;
            gantt.config.auto_scheduling_compatibility = true;

            gantt.config.fit_tasks = true;
            gantt.ext.zoom.init(zoomConfig);
            gantt.ext.zoom.setLevel("day");
            gantt.$zoomToFit = false;
            gantt.i18n.setLocale({
              date: {
                month_full: [Drupal.t("January"), Drupal.t("February"), Drupal.t("March"), Drupal.t("April"), Drupal.t("May"), Drupal.t("June"),
                  Drupal.t("July"), Drupal.t("August"), Drupal.t("September"), Drupal.t("October"), Drupal.t("November"), Drupal.t("December")],
                month_short: [Drupal.t("Jan"), Drupal.t("Feb"), Drupal.t("Mar"), Drupal.t("Apr"), Drupal.t("May"), Drupal.t("Jun"), Drupal.t("Jul"), Drupal.t("Aug"), Drupal.t("Sep"), Drupal.t("Oct"), Drupal.t("Nov"), Drupal.t("Dec")],
                day_full: [Drupal.t("Sunday"), Drupal.t("Monday"), Drupal.t("Tuesday"), Drupal.t("Wednesday"), Drupal.t("Thursday"), Drupal.t("Friday"), Drupal.t("Saturday")],
                day_short: [Drupal.t("Sun"), Drupal.t("Mon"), Drupal.t("Tue"), Drupal.t("Wed"), Drupal.t("Thu"), Drupal.t("Fri"), Drupal.t("Sat")]
              },
              labels: {
                new_task: Drupal.t("New task"),
                icon_save: Drupal.t("Save"),
                icon_cancel: Drupal.t("Cancel"),
                icon_details: Drupal.t("Details"),
                icon_edit: Drupal.t("Edit"),
                icon_delete: Drupal.t("Delete"),
                gantt_save_btn: Drupal.t("New Label"),
                gantt_cancel_btn: Drupal.t("New Label"),
                gantt_delete_btn: Drupal.t("New Label"),
                confirm_deleting: Drupal.t("Task will be deleted permanently, are you sure?"),
                section_description: Drupal.t("Description"),
                section_time: Drupal.t("Time period"),
                section_type: Drupal.t("Type"),

                /* grid columns */
                column_wbs: "WBS",
                column_text: Drupal.t("Task name"),
                column_start_date: Drupal.t("Start time"),
                column_duration: Drupal.t("Duration"),

                /* link confirmation */
                link: Drupal.t("Link"),
                confirm_link_deleting: Drupal.t("will be deleted"),
                link_start: Drupal.t(" (start)"),
                link_end: Drupal.t(" (end)"),

                type_task: Drupal.t("Task"),
                type_project: Drupal.t("Project"),
                type_milestone: Drupal.t("Milestone"),

                minutes: Drupal.t("Minutes"),
                hours: Drupal.t("Hours"),
                days: Drupal.t("Days"),
                weeks: Drupal.t("Week"),
                months: Drupal.t("Months"),
                years: Drupal.t("Years"),

                /* message popup */
                message_ok: Drupal.t("OK"),
                message_cancel: Drupal.t("Cancel"),

                /* constraints */
                section_constraint: Drupal.t("Constraint"),
                constraint_type: Drupal.t("Constraint type"),
                constraint_date: Drupal.t("Constraint date"),
                asap: Drupal.t("As Soon As Possible"),
                alap: Drupal.t("As Late As Possible"),
                snet: Drupal.t("Start No Earlier Than"),
                snlt: Drupal.t("Start No Later Than"),
                fnet: Drupal.t("Finish No Earlier Than"),
                fnlt: Drupal.t("Finish No Later Than"),
                mso: Drupal.t("Must Start On"),
                mfo: Drupal.t("Must Finish On"),

                /* resource control */
                resources_filter_placeholder: Drupal.t("type to filter"),
                resources_filter_label: Drupal.t("hide empty")
              }
            });

            $.each(settings.gantt, function (key, ganttField) {
              let configAjax = {
                url: ganttField.ajax,
                mode: "POST"
              };
              if (ganttField.work_time) {
                gantt.config.work_time = true;
              }
              if (ganttField.show_end) {
                gantt.config.columns.splice(2, 0,{
                  name: "end_date",
                  width: 70,
                  label: Drupal.t("End"),
                  align: "center",
                  template: function (task) {
                    return gantt.templates.date_grid(task.end_date)
                  }
                },);
              }
              if (ganttField.has_planned && !ganttField.split_actual_planned) {
                gantt.config.bar_height = 16;
                gantt.config.row_height = 40;
                gantt.locale.labels.baseline_enable_button = Drupal.t('Set');
                gantt.locale.labels.baseline_disable_button = Drupal.t('Remove');
                gantt.config.lightbox.sections = [
                  {name: Drupal.t("description"), height: 70, map_to: "text", type: "textarea", focus: true},
                  {name: Drupal.t("time"), map_to: "auto", type: "duration"},
                  {
                    name: "baseline",
                    map_to: {start_date: "planned_start", end_date: "planned_end"},
                    button: true,
                    type: "duration_optional"
                  }
                ];
                gantt.locale.labels.section_baseline = Drupal.t("Planned");
                // adding baseline display
                if(gantt.hasOwnProperty('addTaskLayer')){

                  gantt.addTaskLayer({
                    renderer: {
                      render: function draw_planned(task) {
                        if (task.planned_start && task.planned_end) {
                          var sizes = gantt.getTaskPosition(task, task.planned_start, task.planned_end);
                          var el = document.createElement('div');
                          el.className = 'baseline';
                          el.style.left = sizes.left + 'px';
                          el.style.width = sizes.width + 'px';
                          el.style.top = sizes.top + gantt.config.bar_height + 13 + 'px';
                          return el;
                        }
                        return false;
                      },
                      // Define getRectangle in order to hook layer with the smart rendering.
                      getRectangle: function (task, view) {
                        if (task.planned_start && task.planned_end) {
                          return gantt.getTaskPosition(task, task.planned_start, task.planned_end);
                        }
                        return null;
                      }
                    }
                  });

                  gantt.templates.task_class = function (start, end, task) {
                    if (task.planned_end) {
                      var classes = ['has-baseline'];
                      if (end.getTime() > task.planned_end.getTime()) {
                        classes.push('overdue');
                      }
                      return classes.join(' ');
                    }
                  };

                  gantt.templates.rightside_text = function (start, end, task) {
                    if (task.planned_end) {
                      if (end.getTime() > task.planned_end.getTime()) {
                        var overdue = Math.ceil(Math.abs((end.getTime() - task.planned_end.getTime()) / (24 * 60 * 60 * 1000)));
                        var text = "<b>Overdue: " + overdue + " days</b>";
                        return text;
                      }
                    }
                  };

                  gantt.attachEvent("onTaskLoading", function (task) {
                    task.planned_start = gantt.date.parseDate(task.planned_start, "xml_date");
                    task.planned_end = gantt.date.parseDate(task.planned_end, "xml_date");
                    return true;
                  });
                }
              }

              if (ganttField.has_duration) {
                gantt.templates.progress_text = function (start, end, task) {
                  return "<span style='text-align:left;'>" + Math.round(task.progress * 100) + "% </span>";
                };
                gantt.templates.grid_row_class = function (start_date, end_date, item) {
                  if (item.progress == 0) {
                    return "red";
                  }
                  if (item.progress >= 1) {
                    return "green";
                  }
                };
                gantt.templates.task_row_class = function (start_date, end_date, item) {
                  if (item.progress == 0) {
                    return "red";
                  }
                  if (item.progress >= 1) {
                    return "green";
                  }
                };
              }

              if(!ganttField.add_task){
                $.each(gantt.config.columns,function (index, col) {
                  if (col.name === 'add') {
                    delete col.name;
                  }
                });
              }
              if(ganttField.edit_task === false){
                gantt.config.readonly = true;
              }
              if(ganttField.native_dialog){
                native_dialog(ganttField);
              }
              if (isFirst) {
                if ($('#' + ganttField.id).length) {
                  gantt.init(ganttField.id);
                }
                if (typeof dataGantt != "undefined" && dataGantt[ganttField.id] && dataGantt[ganttField.id].data !== undefined) {
                  gantt.parse(dataGantt[ganttField.id]);
                }
                if ($('#planned-' + ganttField.id).length && typeof Gantt != "undefined" && ganttField.split_actual_planned) {
                  Gantt[key] = Gantt.getGanttInstance();
                  Gantt[key].config = gantt.config;
                  Gantt[key].createDataProcessor({
                    url: ganttField.ajax + '?isPlanned=1',
                    mode: "POST"
                  });
                  Gantt[key].init('planned-' + ganttField.id);
                  Gantt[key].parse(dataPlanned[ganttField.id]);
                }

                if (ganttField.owner) {
                  owners(ownerGantt[ganttField.id]);
                }
                if (ganttField.custom_field) {
                  gantt.config.columns.splice(gantt.config.columns.length - 1, 0, {
                    name: ganttField.custom_field, label: ganttField.custom_field_name, align: "center", resize: true
                  });
                  gantt.locale.labels['section_' + ganttField.custom_field] = ganttField.custom_field_name;
                  gantt.config.lightbox.sections.push(
                    {
                      name: ganttField.custom_field,
                      type: "number",
                      map_to: ganttField.custom_field,
                    }
                  );
                  gantt.form_blocks["number"] = {
                    render: function (sns) {
                      return '<input type="number" min="0" class="form-control">';
                    },
                    set_value: function (node, value, task) {
                      node.value = value;
                    },
                    get_value: function (node, task) {
                      return node.value || task.options;
                    },
                    focus: function (node) {
                    }
                  };
                }
                let dp = gantt.createDataProcessor(configAjax);
                dp.init(gantt);
                importMMP(ganttField.id, ganttField.import);
                isFirst = false;
              } else {
                // Multiple Charts on a Page. Unsupported cdn version.
                // it's just show chart, @todo later.
                Gantt[key] = Gantt.getGanttInstance();
                //paragraphGantt[key].config.date_grid = settings.date_format;
                if (typeof dataGantt != "undefined"  && dataGantt[ganttField.id].data !== undefined) {
                  Gantt[key].parse(dataGantt[ganttField.id]);
                }
                let dp = Gantt[key].createDataProcessor(configAjax);
                dp.init(Gantt[key]);
              }
            });
          });
        }
      });
    },
  };
})(jQuery, Drupal, window.Cookies);
